from django.contrib import admin
from image_cropping.admin import ImageCroppingMixin
from models import Image, ImageFK


class ImageFKAdmin(ImageCroppingMixin, admin.ModelAdmin):
    pass


admin.site.register(Image)
admin.site.register(ImageFK, ImageFKAdmin)



import os
BASE_DIR = os.path.dirname(os.path.abspath(__file__))

DEBUG = True
TEMPLATE_DEBUG = True

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite3',
    }
}

TIME_ZONE = 'Europe/Berlin'
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

USE_I18N = True
USE_L10N = True
MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
STATIC_ROOT = os.path.join(BASE_DIR, 'static')
MEDIA_URL = '/media/'
STATIC_URL = '/static/'
SECRET_KEY = '0pfuvtvasdlkjasd76723"b)lna4*f_-xxkszs4##!+wpo'
ROOT_URLCONF = 'example.urls'
TEMPLATE_DIRS = (
    os.path.join(BASE_DIR, 'templates'),
)

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'easy_thumbnails',
    'image_cropping',
    'example',
]

from easy_thumbnails.conf import settings as thumbnail_settings
THUMBNAIL_PROCESSORS = (
    'image_cropping.thumbnail_processors.crop_corners',
) + thumbnail_settings.THUMBNAIL_PROCESSORS


try:
    import django_extensions
except ImportError:
    pass
else:
    INSTALLED_APPS += ['django_extensions',]



from django.shortcuts import render, get_object_or_404
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from easy_thumbnails.files import get_thumbnailer
from .models import Image, ImageFK
from .forms import ImageForm


def thumbnail_options(request):
    try:
        image = Image.objects.all()[0]
    except (Image.DoesNotExist, IndexError,):
        image = None
    return render(request, 'thumbnail_options.html', {'image': image})


def thumbnail_foreign_key(request, instance_id=None):
    if not instance_id:
        try:
            imagefk = ImageFK.objects.all()[0]
        except (ImageFK.DoesNotExist, IndexError,):
            imagefk = None
    else:
        try:
            imagefk = ImageFK.objects.get(pk=instance_id)
        except ImageFK.DoesNotExist:
            imagefk = None
    return render(request, 'thumbnail_foreign_key.html', {'imagefk': imagefk})


def show_thumbnail(request, image_id):
    image = get_object_or_404(Image, pk=image_id)
    thumbnail_url = get_thumbnailer(image.image_field).get_thumbnail({
        'size': (430, 360),
        'box': image.cropping,
        'crop': True,
        'detail': True,
    }).url
    return render(request, 'thumbnail.html', {'thumbnail_url': thumbnail_url})


def modelform_example(request, image_id=None):
    image = get_object_or_404(Image, pk=image_id) if image_id else None
    form = ImageForm(instance=image)
    if request.method == "POST":
        form = ImageForm(request.POST, request.FILES, instance=image)
        if form.is_valid():
            image = form.save()
            return HttpResponseRedirect(reverse('modelform_example', args=(image.pk,)))
    return render(request, 'modelform_example.html', {'form': form, 'image': image})



from django.db import models
from image_cropping.fields import ImageRatioField, ImageCropField


class Image(models.Model):
    image_field = ImageCropField(upload_to='image/')
    cropping = ImageRatioField('image_field', '120x100')

    class Meta:
        app_label = 'example'


class ImageFK(models.Model):
    image = models.ForeignKey(Image)
    cropping = ImageRatioField('image__image_field', '120x100')

    class Meta:
        app_label = 'example'



from django import template
from easy_thumbnails.files import get_thumbnailer
from easy_thumbnails.exceptions import InvalidImageFormatError

register = template.Library()

# Sytanx:
# {% cropped_thumbnail instancename ratiofieldname [scale=0.1|width=100|height=200] [upscale] %}
@register.tag
def cropped_thumbnail(parser, token):
    args = token.split_contents()

    if len(args) < 3:
        # requites model and ratiofieldname
        raise template.TemplateSyntaxError("%r tag requires at least two arguments" % args[0])

    option = None
    upscale = False

    instance = args[1]
    # strip quotes from ratio field
    ratiofieldname = args[2].strip('"\'')

    # parse additional arguments
    for arg in args[3:]:
        arg = arg.lower()
        try:
            name, value = arg.split('=')

            if option:
                raise template.TemplateSyntaxError("%s: there is already an option defined!" % arg)
            try: # parse option
                option = (name, float(value))
                if not option[0] in ('scale', 'width', 'height'):
                    raise template.TemplateSyntaxError("invalid optional argument %s" % args[3])
                if option[1] < 0:
                    raise template.TemplateSyntaxError("%s must have a positive value" % option[0])
            except ValueError:
                raise template.TemplateSyntaxError("%s needs an numeric argument" % args[3])

        except ValueError: # check for upscale argument
            if arg == 'upscale':
                upscale = True
            else:
                raise template.TemplateSyntaxError("%s is an invalid option" % arg)

    return CroppingNode(instance, ratiofieldname, option, upscale)


class CroppingNode(template.Node):
    def __init__(self, instance, ratiofieldname, option=None, upscale=False):
        self.instance = instance
        self.ratiofieldname = ratiofieldname
        self.option = option
        self.upscale = upscale

    def render(self, context):
        instance = template.Variable(self.instance).resolve(context)
        if not instance:
            return

        ratiofield = instance._meta.get_field(self.ratiofieldname)
        image = getattr(instance, ratiofield.image_field) # get imagefield

        if ratiofield.image_fk_field: # image is ForeignKey
            # get the imagefield
            image = getattr(image, ratiofield.image_fk_field)

        size = (int(ratiofield.width), int(ratiofield.height))
        box = getattr(instance, self.ratiofieldname)

        option = self.option
        if option:
            if option[0] == 'scale':
                width = size[0] * option[1]
                height = size[1] * option[1]
            elif option[0] == 'width':
                width = option[1]
                height = size[1] * width / size[0]
            elif option[0] == 'height':
                height = option[1]
                width = height * size[0] / size[1]
            size = (int(width), int(height))

        if ratiofield.adapt_rotation:
            if (image.height > image.width) != (size[1] > size[0]):
                # box needs rotation
                size = (size[1], size[0])

        thumbnailer = get_thumbnailer(image)
        thumbnail_options = {
            'size': size,
            'box': box,
            'crop': True,
            'detail': True,
            'upscale': self.upscale
        }
        try:
            thumb = thumbnailer.get_thumbnail(thumbnail_options)
        except InvalidImageFormatError:
            return ''
        return thumb.url



from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.utils import override_settings

from example.models import ImageFK
from .factory import create_cropped_image, create_superuser

TEST_CROPPING = (355,355)

class CroppingTestCase(TestCase):

    def test_image_cropping(self):
        """Test if the tumbnail for a cropped image gets generated with the correct box parameters."""
        image = create_cropped_image()
        response = self.client.get(reverse('show_thumbnail', args=(image.pk,)))
        self.assertEqual(response.context['thumbnail_url'].split('.')[1], '430x360_q85_box-51%2C53%2C151%2C136_crop_detail')

    def test_image_fk_cropping(self):
        """Test if the thumbnail for CropForeignKey fields gets generated correctly."""
        image = create_cropped_image()
        example = ImageFK.objects.create(image=image, cropping=image.cropping)
        response = self.client.get(reverse('thumbnail_foreign_key', args=(example.pk,)))
        self.assertContains(response, '120x100_q85_box-51%2C53%2C151%2C136_crop_detail')

    @override_settings(IMAGE_CROPPING_THUMB_SIZE=TEST_CROPPING)
    def test_override_image_thumb_size(self):
        """Test if the IMAGE_CROPPING_THUMB_SIZE setting is respected."""
        admin = create_superuser()
        image = create_cropped_image()
        self.client.login(username=admin.username, password='admin')
        response = self.client.get(reverse('admin:example_image_change', args=[image.pk,]))
        test_url = 'data-thumbnail-url="%s.%sx%s' % (image.image_field.url, TEST_CROPPING[0], TEST_CROPPING[1])
        self.assertContains(response, test_url)



import logging
import inspect
import warnings

from django.db.models import get_model, ObjectDoesNotExist
from django import forms
from django.contrib.admin.widgets import AdminFileWidget, ForeignKeyRawIdWidget
from django.conf import settings

from easy_thumbnails.files import get_thumbnailer

logger = logging.getLogger(__name__)


def thumbnail(image_path):
    thumbnailer = get_thumbnailer(image_path)
    thumbnail_options = {
        'detail': True,
        'size': getattr(settings, 'IMAGE_CROPPING_THUMB_SIZE', (300, 300)),
    }
    thumb = thumbnailer.get_thumbnail(thumbnail_options)
    return thumb.url


def get_attrs(image, name):
    try:
        return {
            'class': "crop-thumb",
            'data-thumbnail-url': thumbnail(image),
            'data-field-name': name,
            'data-org-width': image.width,
            'data-org-height': image.height,
        }
    except ValueError:
        # can't create thumbnail from image
        return {}


class CropWidget(object):
    class Media:
        js = (
            getattr(settings, 'JQUERY_URL',
                'https://ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min.js'),
            "image_cropping/js/jquery.Jcrop.min.js",
            "image_cropping/image_cropping.js",
        )
        css= {'all' : ("image_cropping/css/jquery.Jcrop.min.css",)}


class ImageCropWidget(AdminFileWidget, CropWidget):
    def render(self, name, value, attrs=None):
        if not attrs:
            attrs = {}
        if value:
            attrs.update(get_attrs(value, name))
        return super(AdminFileWidget, self).render(name, value, attrs)


class HiddenImageCropWidget(forms.HiddenInput, CropWidget):
    def render(self, name, value, attrs=None):
        if not attrs:
            attrs = {}
        # we need to hide it the whole field by JS because the admin
        # doesn't yet support hidden fields:
        # https://code.djangoproject.com/ticket/11277
        attrs['data-hide-field'] = True
        if value:
            attrs.update(get_attrs(value, name))
        return super(HiddenImageCropWidget, self).render(name, value, attrs)


class CropForeignKeyWidget(ForeignKeyRawIdWidget, CropWidget):
    def __init__(self, *args, **kwargs):
        self.field_name = kwargs.pop('field_name')
        # Django versions 1.4+ need the admin site passed in
        if 'admin_site' in inspect.getargspec(ForeignKeyRawIdWidget.__init__)[0]:
            # Django 1.4+
            if 'admin_site' not in kwargs:
                warnings.warn('Please use the ImageCroppingMixin in your ModelAdmin '
                              'instead of the CropForeignKey.', DeprecationWarning)
                from django.contrib.admin.sites import site
                kwargs['admin_site'] = site
        elif 'admin_site' in kwargs:
            # Django < 1.4 and admin_site passed in from ImageCroppingMixin
            del kwargs['admin_site']

        super(CropForeignKeyWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        if attrs is None:
            attrs = {}

        if value:
            app_name = self.rel.to._meta.app_label
            model_name = self.rel.to._meta.object_name.lower()
            try:
                image = getattr(get_model(app_name, model_name
                    ).objects.get(pk=value), self.field_name)
                attrs.update(get_attrs(image, name))
            except ObjectDoesNotExist:
                logger.error("Can't find object: %s.%s with primary key %s "
                    "for cropping." % (app_name, model_name, value))
            except AttributeError:
                logger.error("Object %s.%s doesn't have an attribute named '%s'." % (
                    app_name, model_name, self.field_name))
        return super(CropForeignKeyWidget, self).render(name, value, attrs)



import warnings
from django.db import models
from django import forms
from django.conf import settings
from .widgets import ImageCropWidget


class ImageCropField(models.ImageField):
    def formfield(self, *args, **kwargs):
        kwargs['widget'] = ImageCropWidget
        return super(ImageCropField, self).formfield(*args, **kwargs)

    def south_field_triple(self):
        """
        Return a suitable description of this field for South.
        """
        # We'll just introspect ourselves, since we inherit.
        from south.modelsinspector import introspector
        field_class = "django.db.models.fields.files.ImageField"
        args, kwargs = introspector(self)
        return (field_class, args, kwargs)


#deprecated, as we now set the widget in the ModelAdmin
class CropForeignKey(models.ForeignKey):
    '''
    A croppable image field contained in another model. Only works in admin
    for now, as it uses the raw_id widget.
    '''

    def __init__(self, model, field_name, *args, **kwargs):
        self.field_name = field_name
        warnings.warn('Please use the ImageCroppingMixin in your ModelAdmin '
                      'instead of a CropForeignKey!', DeprecationWarning)
        super(CropForeignKey, self).__init__(model, *args, **kwargs)

    def south_field_triple(self):
        """
        Return a suitable description of this field for South.
        """
        # We'll just introspect ourselves, since we inherit.
        from south.modelsinspector import introspector
        field_class = "django.db.models.fields.related.ForeignKey"
        args, kwargs = introspector(self)
        return (field_class, args, kwargs)


class ImageRatioField(models.CharField):
    def __init__(self, image_field, size, adapt_rotation=False,
                 allow_fullsize=False, verbose_name=None, help_text=None,
                 hide_image_field=False, size_warning=getattr(
                     settings, 'IMAGE_CROPPING_SIZE_WARNING', False)):
        if '__' in image_field:
            self.image_field, self.image_fk_field = image_field.split('__')
        else:
            self.image_field, self.image_fk_field = image_field, None
        self.width, self.height = size.split('x')
        self.adapt_rotation = adapt_rotation
        self.allow_fullsize = allow_fullsize
        self.size_warning = size_warning
        self.hide_image_field = hide_image_field
        super(ImageRatioField, self).__init__(max_length=255, blank=True, verbose_name=verbose_name, help_text=help_text)

    def contribute_to_class(self, cls, name):
        super(ImageRatioField, self).contribute_to_class(cls, name)
        # attach a list of fields that are referenced by the ImageRatioField
        # so we can set the correct widget in the ModelAdmin
        if not hasattr(cls, 'crop_fields'):
            cls.add_to_class('crop_fields', {})
        cls.crop_fields[self.image_field] = {
            'fk_field': self.image_fk_field,
            'hidden': self.hide_image_field,
        }

    def formfield(self, *args, **kwargs):
        kwargs['widget'] = forms.TextInput(attrs={
            'data-width': int(self.width),
            'data-height': int(self.height),
            'data-image-field': self.image_field,
            'data-my-name': self.name,
            'data-adapt-rotation': str(self.adapt_rotation).lower(),
            'data-allow-fullsize': str(self.allow_fullsize).lower(),
            'data-size-warning': str(self.size_warning).lower(),
            'class': 'image-ratio',
        })
        return super(ImageRatioField, self).formfield(*args, **kwargs)

    def south_field_triple(self):
        """
        Return a suitable description of this field for South.
        """
        # We'll just introspect ourselves, since we inherit.
        from south.modelsinspector import introspector
        field_class = "django.db.models.fields.CharField"
        args, kwargs = introspector(self)
        return (field_class, args, kwargs)



from django.core.urlresolvers import reverse
from django.test import LiveServerTestCase
from selenium.webdriver.firefox.webdriver import WebDriver
from selenium.webdriver.support.ui import WebDriverWait
from .factory import create_superuser, create_cropped_image

class BrowserTestCase(LiveServerTestCase):

    @classmethod
    def setUpClass(cls):
        cls.selenium = WebDriver()
        super(BrowserTestCase, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super(BrowserTestCase, cls).tearDownClass()

    def login(self):
        create_superuser()
        self.selenium.get('%s%s' % (self.live_server_url, '/admin'))
        username_input = self.selenium.find_element_by_id("id_username")
        password_input = self.selenium.find_element_by_id("id_password")
        username_input.send_keys('admin')
        password_input.send_keys('admin')
        self.selenium.find_element_by_xpath('//input[@value="Log in"]').click()
        WebDriverWait(self.selenium, 10)

    def test_admin_cropping(self):
        """Test if the thumbnail for cropping images gets correctly embedded in the admin."""
        image = create_cropped_image()
        self.login()
        edit_view = reverse('admin:example_image_change', args=[image.pk,])
        self.selenium.get('%s%s' % (self.live_server_url, edit_view))
        WebDriverWait(self.selenium, 10)
        thumbnail = self.selenium.find_element_by_css_selector('.jcrop-holder img')
        thumbnail_source = thumbnail.get_attribute('src')
        self.assertTrue(image.image_field.url in thumbnail_source)

    def test_modelform_cropping(self):
        """Test if the thumbnail for cropping images gets correctly embedded when using modelforms."""
        image = create_cropped_image()
        edit_view = reverse('modelform_example', args=[image.pk,])
        self.selenium.get('%s%s' % (self.live_server_url, edit_view))
        WebDriverWait(self.selenium, 10)
        thumbnail = self.selenium.find_element_by_css_selector('.jcrop-holder img')
        thumbnail_source = thumbnail.get_attribute('src')
        self.assertTrue(image.image_field.url in thumbnail_source)



# This file mainly exists to allow python setup.py test to work.
import os, sys
os.environ['DJANGO_SETTINGS_MODULE'] = 'example.settings'
test_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, test_dir)

from django.test.utils import get_runner
from django.conf import settings

def runtests():
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=1, interactive=True)
    failures = test_runner.run_tests(['example'])
    sys.exit(bool(failures))

if __name__ == '__main__':
    runtests()



from django.core.files import File
from django.conf import settings
from django.contrib.auth.models import User
from example.models import Image

def create_cropped_image(**kwargs):
    defaults = {
        'image_cropping':  u'51,53,151,136',
        'image_path': settings.STATIC_ROOT + "/images/example_image.jpg",
        'image_name': 'example_image',
    }
    defaults.update(kwargs)
    image = Image.objects.create(**{'cropping': defaults['image_cropping']})
    image.image_field.save(defaults['image_name'], File(open(defaults['image_path'])), True)
    return image


def create_superuser(**kwargs):
    defaults = {
        'password': 'admin',
        'username': 'admin',
        'email': 'admin@admin.test',
    }
    defaults.update(kwargs)
    return User.objects.create_superuser(**defaults)





from django.conf.urls import patterns, include, url, static
from django.conf import settings

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    url(r'^$', 'example.views.thumbnail_options', name='thumbnail_options'),
    url(r'^show_foreignkey_thumbnail/$', 'example.views.thumbnail_foreign_key', name='thumbnail_foreign_key'),
    url(r'^show_foreignkey_thumbnail/(?P<instance_id>\d+)/$', 'example.views.thumbnail_foreign_key', name='thumbnail_foreign_key'),
    url(r'^modelform_example/(?P<image_id>\d+)/$', 'example.views.modelform_example', name='modelform_example'),
    url(r'^modelform_example/$', 'example.views.modelform_example', name='modelform_example'),
    url(r'^show_thumbnail/(?P<image_id>\d+)/$', 'example.views.show_thumbnail', name='show_thumbnail'),
    url(r'^admin/', include(admin.site.urls)),
)
urlpatterns += static.static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



import logging


logger = logging.getLogger(__name__)

def crop_corners(image, box=None, **kwargs):
    """
    Crop corners to the selection defined by image_cropping
    """

    if box and box[0] != '-':
        try:
            values = [int(x) for x in box.split(',')]
            if sum(values) < 0:
                return image
            width = abs(values[2] - values[0])
            height = abs(values[3] - values[1])
            if width and height and (width != image.size[0] or height != image.size[1]):
                image = image.crop(values)
        except (ValueError, IndexError):
            # There's garbage in the cropping field, ignore
            logger.warning('Unable to parse "box" parameter value "%s". Ignoring.' % box)

    return image

